@@ -17,13 +17,18 @@
 // [[Rcpp::export]]
 Eigen::MatrixXd rcppeigen_hello_world() {
     Eigen::MatrixXd m1 = Eigen::MatrixXd::Identity(3, 3);
-    Eigen::MatrixXd m2 = Eigen::MatrixXd::Random(3, 3);
-	                     
+    // Eigen::MatrixXd m2 = Eigen::MatrixXd::Random(3, 3);
+    // Do not use Random() here to not promote use of a non-R RNG
+    Eigen::MatrixXd m2 = Eigen::MatrixXd::Zero(3, 3);
+    for (auto i=0; i<m2.rows(); i++)
+        for (auto j=0; j<m2.cols(); j++)
+            m2(i,j) = R::rnorm(0, 1);
+
     return m1 + 3 * (m1 + m2);
 }
 
 
-// another simple example: outer product of a vector, 
+// another simple example: outer product of a vector,
 // returning a matrix
 //
 // [[Rcpp::export]]
